@@ -1,3 +1,8 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff;
 
 import java.io.OutputStreamWriter;
@@ -16,7 +21,6 @@ public class Main {
      * Creates a new Main object.
      */
     private Main() {
-        super();
     }
 
     /**
@@ -29,14 +33,16 @@ public class Main {
      */
     public static void main(final String[] args)
             throws UnsupportedEncodingException {
+        @SuppressWarnings("UseOfSystemOutOrSystemErr")
         final PrintWriter writer = new PrintWriter(System.out, true);
         final PgDiffArguments arguments = new PgDiffArguments();
 
         if (arguments.parse(writer, args)) {
+            @SuppressWarnings("UseOfSystemOutOrSystemErr")
             final PrintWriter encodedWriter = new PrintWriter(
                     new OutputStreamWriter(
                     System.out, arguments.getOutCharsetName()));
-            PgDiff.createDiff(encodedWriter, arguments);
+                PgDiff.createDiff(encodedWriter, arguments);
             encodedWriter.close();
         }
 
