@@ -1,9 +1,13 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff;
 
-import java.io.BufferedReader;
-import java.io.IOException;
-import java.io.InputStreamReader;
 import java.io.PrintWriter;
+import java.nio.charset.Charset;
+import java.util.SortedMap;
 
 /**
  * Contains parsed command line arguments.
@@ -34,7 +38,7 @@ public class PgDiffArguments {
      */
     private boolean addDefaults;
     /**
-     * Whether to enclose all commands in transaction.
+     * Whether to enclose all statements in transaction.
      */
     private boolean addTransaction;
     /**
@@ -45,14 +49,18 @@ public class PgDiffArguments {
      * Whether to ignore START WITH on SEQUENCEs.
      */
     private boolean ignoreStartWith;
-    /**
-     * Whether to quote names when creating the diff SQL commands.
-     */
-    private boolean quoteNames;
     /**
      * Whether to display apgdiff version.
      */
     private boolean version;
+    /**
+     * Whether to output information about ignored statements.
+     */
+    private boolean outputIgnoredStatements;
+    /**
+     * Whether to list supported charsets.
+     */
+    private boolean listCharsets;
 
     /**
      * Setter for {@link #addDefaults}.
@@ -164,21 +172,22 @@ public class PgDiffArguments {
     }
 
     /**
-     * Setter for {@link #quoteNames}.
+     * Getter for {@link #outputIgnoredStatements}.
      *
-     * @param quoteNames {@link #quoteNames}
+     * @return {@link #outputIgnoredStatements}
      */
-    public void setQuoteNames(final boolean quoteNames) {
-        this.quoteNames = quoteNames;
+    public boolean isOutputIgnoredStatements() {
+        return outputIgnoredStatements;
     }
 
     /**
-     * Getter for {@link #quoteNames}.
+     * Setter for {@link #outputIgnoredStatements}.
      *
-     * @return {@link #quoteNames}
+     * @param outputIgnoredStatements {@link #outputIgnoredStatements}
      */
-    public boolean isQuoteNames() {
-        return quoteNames;
+    public void setOutputIgnoredStatements(
+            final boolean outputIgnoredStatements) {
+        this.outputIgnoredStatements = outputIgnoredStatements;
     }
 
     /**
@@ -230,24 +239,31 @@ public class PgDiffArguments {
             } else if ("--in-charset-name".equals(args[i])) {
                 setInCharsetName(args[i + 1]);
                 i++;
+            } else if ("--list-charsets".equals(args[i])) {
+                setListCharsets(true);
             } else if ("--out-charset-name".equals(args[i])) {
                 setOutCharsetName(args[i + 1]);
                 i++;
-            } else if ("--quote-names".equals(args[i])) {
-                setQuoteNames(true);
+            } else if ("--output-ignored-statements".equals(args[i])) {
+                setOutputIgnoredStatements(true);
             } else if ("--version".equals(args[i])) {
                 setVersion(true);
             } else {
-                writer.println("ERROR: Unknown option: " + args[i]);
+                writer.print(Resources.getString("ErrorUnknownOption"));
+                writer.print(": ");
+                writer.println(args[i]);
                 success = false;
 
                 break;
             }
         }
 
-        if ((args.length == 1) && isVersion()) {
+        if (args.length == 1 && isVersion()) {
             printVersion(writer);
             success = false;
+        } else if (args.length == 1 && isListCharsets()) {
+            listCharsets(writer);
+            success = false;
         } else if (args.length < 2) {
             printUsage(writer);
             success = false;
@@ -263,66 +279,21 @@ public class PgDiffArguments {
      * Prints program usage.
      *
      * @param writer writer to print the usage to
-     *
-     * @throws RuntimeException Thrown if problem occured while reading usage
-     *         info.
      */
     private void printUsage(final PrintWriter writer) {
-        final BufferedReader reader =
-                new BufferedReader(
-                new InputStreamReader(
-                getClass().getResourceAsStream("usage.txt")));
-
-        try {
-            String line = reader.readLine();
-
-            while (line != null) {
-                writer.println(line);
-                line = reader.readLine();
-            }
-        } catch (final IOException ex) {
-            throw new RuntimeException(
-                    "Problem occured while reading usage file",
-                    ex);
-        } finally {
-            try {
-                reader.close();
-            } catch (final IOException ex) {
-                throw new RuntimeException(
-                        "Problem occured while closing reader",
-                        ex);
-            }
-        }
+        writer.println(
+                Resources.getString("UsageHelp").replace("${tab}", "\t"));
     }
 
     /**
      * Prints program version.
      *
      * @param writer writer to print the usage to
-     *
-     * @throws RuntimeException Thrown if problem occured while reading program
-     *         version.
      */
     private void printVersion(final PrintWriter writer) {
-        final BufferedReader reader =
-                new BufferedReader(
-                new InputStreamReader(
-                getClass().getResourceAsStream("build_info")));
-        writer.print("Version: ");
-
-        try {
-            writer.println(reader.readLine());
-        } catch (final IOException ex) {
-            throw new RuntimeException("Cannot read program version", ex);
-        } finally {
-            try {
-                reader.close();
-            } catch (final IOException ex) {
-                throw new RuntimeException(
-                        "Problem occured while closing reader",
-                        ex);
-            }
-        }
+        writer.print(Resources.getString("Version"));
+        writer.print(": ");
+        writer.println(Resources.getString("VersionNumber"));
     }
 
     /**
@@ -360,4 +331,35 @@ public class PgDiffArguments {
     public void setOutCharsetName(final String outCharsetName) {
         this.outCharsetName = outCharsetName;
     }
+
+    /**
+     * Getter for {@link #listCharsets}.
+     *
+     * @return {@link #listCharsets}
+     */
+    public boolean isListCharsets() {
+        return listCharsets;
+    }
+
+    /**
+     * Setter for {@link #listCharsets}.
+     *
+     * @param listCharsets {@link #listCharsets}
+     */
+    public void setListCharsets(final boolean listCharsets) {
+        this.listCharsets = listCharsets;
+    }
+
+    /**
+     * Lists supported charsets.
+     *
+     * @param writer writer
+     */
+    private void listCharsets(final PrintWriter writer) {
+        final SortedMap<String, Charset> charsets = Charset.availableCharsets();
+
+        for (final String name : charsets.keySet()) {
+            writer.println(name);
+        }
+    }
 }
