@@ -1,3 +1,8 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff;
 
 import cz.startnet.utils.pgdiff.schema.PgConstraint;
@@ -20,22 +25,21 @@ public class PgDiffConstraints {
      * Creates a new instance of PgDiffConstraints.
      */
     private PgDiffConstraints() {
-        super();
     }
 
     /**
-     * Outputs commands for creation of new constraints.
+     * Outputs statements for creation of new constraints.
      *
      * @param writer writer the output should be written to
-     * @param arguments object containing arguments settings
      * @param oldSchema original schema
      * @param newSchema new schema
-     * @param primaryKey determines whether primery keys should be processed or
+     * @param primaryKey determines whether primary keys should be processed or
      *        any other constraints should be processed
+     * @param searchPathHelper search path helper
      */
     public static void createConstraints(final PrintWriter writer,
-            final PgDiffArguments arguments, final PgSchema oldSchema,
-            final PgSchema newSchema, final boolean primaryKey) {
+            final PgSchema oldSchema, final PgSchema newSchema,
+            final boolean primaryKey, final SearchPathHelper searchPathHelper) {
         for (final PgTable newTable : newSchema.getTables()) {
             final PgTable oldTable;
 
@@ -48,26 +52,26 @@ public class PgDiffConstraints {
             // Add new constraints
             for (final PgConstraint constraint :
                     getNewConstraints(oldTable, newTable, primaryKey)) {
+                searchPathHelper.outputSearchPath(writer);
                 writer.println();
-                writer.println(
-                        constraint.getCreationSQL(arguments.isQuoteNames()));
+                writer.println(constraint.getCreationSQL());
             }
         }
     }
 
     /**
-     * Outputs commands for dropping non-existant or modified constraints.
+     * Outputs statements for dropping non-existent or modified constraints.
      *
      * @param writer writer the output should be written to
-     * @param arguments object containing arguments settings
      * @param oldSchema original schema
      * @param newSchema new schema
-     * @param primaryKey determines whether primery keys should be processed or
+     * @param primaryKey determines whether primary keys should be processed or
      *        any other constraints should be processed
+     * @param searchPathHelper search path helper
      */
     public static void dropConstraints(final PrintWriter writer,
-            final PgDiffArguments arguments, final PgSchema oldSchema,
-            final PgSchema newSchema, final boolean primaryKey) {
+            final PgSchema oldSchema, final PgSchema newSchema,
+            final boolean primaryKey, final SearchPathHelper searchPathHelper) {
         for (final PgTable newTable : newSchema.getTables()) {
             final PgTable oldTable;
 
@@ -80,8 +84,9 @@ public class PgDiffConstraints {
             // Drop constraints that no more exist or are modified
             for (final PgConstraint constraint :
                     getDropConstraints(oldTable, newTable, primaryKey)) {
+                searchPathHelper.outputSearchPath(writer);
                 writer.println();
-                writer.println(constraint.getDropSQL(arguments.isQuoteNames()));
+                writer.println(constraint.getDropSQL());
             }
         }
     }
@@ -91,7 +96,7 @@ public class PgDiffConstraints {
      *
      * @param oldTable original table or null
      * @param newTable new table or null
-     * @param primaryKey determines whether primery keys should be processed or
+     * @param primaryKey determines whether primary keys should be processed or
      *        any other constraints should be processed
      *
      * @return list of constraints that should be dropped
@@ -101,11 +106,12 @@ public class PgDiffConstraints {
      */
     private static List<PgConstraint> getDropConstraints(final PgTable oldTable,
             final PgTable newTable, final boolean primaryKey) {
+        @SuppressWarnings("CollectionWithoutInitialCapacity")
         final List<PgConstraint> list = new ArrayList<PgConstraint>();
 
-        if ((newTable != null) && (oldTable != null)) {
+        if (newTable != null && oldTable != null) {
             for (final PgConstraint constraint : oldTable.getConstraints()) {
-                if ((constraint.isPrimaryKeyConstraint() == primaryKey)
+                if (constraint.isPrimaryKeyConstraint() == primaryKey
                         && (!newTable.containsConstraint(constraint.getName())
                         || !newTable.getConstraint(constraint.getName()).equals(
                         constraint))) {
@@ -122,13 +128,14 @@ public class PgDiffConstraints {
      *
      * @param oldTable original table
      * @param newTable new table
-     * @param primaryKey determines whether primery keys should be processed or
+     * @param primaryKey determines whether primary keys should be processed or
      *        any other constraints should be processed
      *
      * @return list of constraints that should be added
      */
     private static List<PgConstraint> getNewConstraints(final PgTable oldTable,
             final PgTable newTable, final boolean primaryKey) {
+        @SuppressWarnings("CollectionWithoutInitialCapacity")
         final List<PgConstraint> list = new ArrayList<PgConstraint>();
 
         if (newTable != null) {
@@ -155,4 +162,85 @@ public class PgDiffConstraints {
 
         return list;
     }
+
+    /**
+     * Outputs statements for constraint comments that have changed.
+     *
+     * @param writer writer
+     * @param oldSchema old schema
+     * @param newSchema new schema
+     * @param searchPathHelper search path helper
+     */
+    public static void alterComments(final PrintWriter writer,
+            final PgSchema oldSchema, final PgSchema newSchema,
+            final SearchPathHelper searchPathHelper) {
+        if (oldSchema == null) {
+            return;
+        }
+
+        for (PgTable oldTable : oldSchema.getTables()) {
+            final PgTable newTable = newSchema.getTable(oldTable.getName());
+
+            if (newTable == null) {
+                continue;
+            }
+
+            for (final PgConstraint oldConstraint : oldTable.getConstraints()) {
+                final PgConstraint newConstraint =
+                        newTable.getConstraint(oldConstraint.getName());
+
+                if (newConstraint == null) {
+                    continue;
+                }
+
+                if (oldConstraint.getComment() == null
+                        && newConstraint.getComment() != null
+                        || oldConstraint.getComment() != null
+                        && newConstraint.getComment() != null
+                        && !oldConstraint.getComment().equals(
+                        newConstraint.getComment())) {
+                    searchPathHelper.outputSearchPath(writer);
+                    writer.println();
+                    writer.print("COMMENT ON ");
+
+                    if (newConstraint.isPrimaryKeyConstraint()) {
+                        writer.print("INDEX ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getName()));
+                    } else {
+                        writer.print("CONSTRAINT ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getName()));
+                        writer.print(" ON ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getTableName()));
+                    }
+
+                    writer.print(" IS ");
+                    writer.print(newConstraint.getComment());
+                    writer.println(';');
+                } else if (oldConstraint.getComment() != null
+                        && newConstraint.getComment() == null) {
+                    searchPathHelper.outputSearchPath(writer);
+                    writer.println();
+                    writer.print("COMMENT ON ");
+
+                    if (newConstraint.isPrimaryKeyConstraint()) {
+                        writer.print("INDEX ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getName()));
+                    } else {
+                        writer.print("CONSTRAINT ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getName()));
+                        writer.print(" ON ");
+                        writer.print(PgDiffUtils.getQuotedName(
+                                newConstraint.getTableName()));
+                    }
+
+                    writer.println(" IS NULL;");
+                }
+            }
+        }
+    }
 }
