@@ -1,23 +1,27 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.loader;
 
 /**
- * Exception thrown if problem occured while reading or writing file.
+ * Exception thrown if problem occurred while reading or writing file.
  *
  * @author fordfrog
  */
 public class FileException extends RuntimeException {
 
     /**
-     * Predefined message that can be used when cannot read file.
+     * Serial version uid.
      */
-    public static final String CANNOT_READ_FILE = "Cannot read file";
+    private static final long serialVersionUID = 1L;
 
     /**
      * Creates a new instance of <code>FileException</code> without
      * detail message.
      */
     public FileException() {
-        super();
     }
 
     /**
