@@ -1,6 +1,14 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.loader;
 
+import cz.startnet.utils.pgdiff.Resources;
 import cz.startnet.utils.pgdiff.parsers.AlterTableParser;
+import cz.startnet.utils.pgdiff.parsers.AlterViewParser;
+import cz.startnet.utils.pgdiff.parsers.CommentParser;
 import cz.startnet.utils.pgdiff.parsers.CreateFunctionParser;
 import cz.startnet.utils.pgdiff.parsers.CreateIndexParser;
 import cz.startnet.utils.pgdiff.parsers.CreateSchemaParser;
@@ -17,6 +25,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
+import java.text.MessageFormat;
 
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -29,7 +38,7 @@ import java.util.regex.Pattern;
 public class PgDumpLoader { //NOPMD
 
     /**
-     * Pattern for testing whether command is CREATE SCHEMA command.
+     * Pattern for testing whether it is CREATE SCHEMA statement.
      */
     private static final Pattern PATTERN_CREATE_SCHEMA = Pattern.compile(
             "^CREATE[\\s]+SCHEMA[\\s]+.*$", Pattern.CASE_INSENSITIVE);
@@ -41,86 +50,82 @@ public class PgDumpLoader { //NOPMD
             "^SET[\\s]+search_path[\\s]*=[\\s]*([^,\\s]+)(?:,[\\s]+.*)?;$",
             Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE TABLE command.
+     * Pattern for testing whether it is CREATE TABLE statement.
      */
     private static final Pattern PATTERN_CREATE_TABLE = Pattern.compile(
             "^CREATE[\\s]+TABLE[\\s]+.*$", Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE VIEW command.
+     * Pattern for testing whether it is CREATE VIEW statement.
      */
     private static final Pattern PATTERN_CREATE_VIEW = Pattern.compile(
             "^CREATE[\\s]+(?:OR[\\s]+REPLACE[\\s]+)?VIEW[\\s]+.*$",
             Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is ALTER TABLE command.
+     * Pattern for testing whether it is ALTER TABLE statement.
      */
     private static final Pattern PATTERN_ALTER_TABLE =
-            Pattern.compile("^ALTER[\\s]+TABLE[\\s]+.*$", Pattern.CASE_INSENSITIVE);
+            Pattern.compile("^ALTER[\\s]+TABLE[\\s]+.*$",
+            Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE SEQUENCE command.
+     * Pattern for testing whether it is CREATE SEQUENCE statement.
      */
     private static final Pattern PATTERN_CREATE_SEQUENCE = Pattern.compile(
             "^CREATE[\\s]+SEQUENCE[\\s]+.*$", Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE INDEX command.
+     * Pattern for testing whether it is CREATE INDEX statement.
      */
     private static final Pattern PATTERN_CREATE_INDEX = Pattern.compile(
             "^CREATE[\\s]+(?:UNIQUE[\\s]+)?INDEX[\\s]+.*$",
             Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is SET command.
-     */
-    private static final Pattern PATTERN_SET =
-            Pattern.compile("^SET[\\s]+.*$", Pattern.CASE_INSENSITIVE);
-    /**
-     * Pattern for testing whether command is COMMENT command.
-     */
-    private static final Pattern PATTERN_COMMENT =
-            Pattern.compile("^COMMENT[\\s]+.*$", Pattern.CASE_INSENSITIVE);
-    /**
-     * Pattern for testing whether command is SELECT command.
+     * Pattern for testing whether it is SELECT statement.
      */
     private static final Pattern PATTERN_SELECT =
             Pattern.compile("^SELECT[\\s]+.*$", Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is INSERT INTO command.
+     * Pattern for testing whether it is INSERT INTO statement.
      */
     private static final Pattern PATTERN_INSERT_INTO =
-            Pattern.compile("^INSERT[\\s]+INTO[\\s]+.*$", Pattern.CASE_INSENSITIVE);
+            Pattern.compile("^INSERT[\\s]+INTO[\\s]+.*$",
+            Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is REVOKE command.
+     * Pattern for testing whether it is UPDATE statement.
      */
-    private static final Pattern PATTERN_REVOKE =
-            Pattern.compile("^REVOKE[\\s]+.*$", Pattern.CASE_INSENSITIVE);
+    private static final Pattern PATTERN_UPDATE =
+            Pattern.compile("^UPDATE[\\s].*$", Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is GRANT command.
+     * Pattern for testing whether it is DELETE FROM statement.
      */
-    private static final Pattern PATTERN_GRANT =
-            Pattern.compile("^GRANT[\\s]+.*$", Pattern.CASE_INSENSITIVE);
+    private static final Pattern PATTERN_DELETE_FROM =
+            Pattern.compile("^DELETE[\\s]+FROM[\\s]+.*$",
+            Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE TRIGGER command.
+     * Pattern for testing whether it is CREATE TRIGGER statement.
      */
     private static final Pattern PATTERN_CREATE_TRIGGER = Pattern.compile(
             "^CREATE[\\s]+TRIGGER[\\s]+.*$", Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for testing whether command is CREATE FUNCTION or CREATE
-     * OR REPLACE FUNCTION command.
+     * Pattern for testing whether it is CREATE FUNCTION or CREATE OR REPLACE
+     * FUNCTION statement.
      */
     private static final Pattern PATTERN_CREATE_FUNCTION = Pattern.compile(
             "^CREATE[\\s]+(?:OR[\\s]+REPLACE[\\s]+)?FUNCTION[\\s]+.*$",
             Pattern.CASE_INSENSITIVE);
     /**
-     * Pattern for getting the string that is used to end the function
-     * or the function definition itself.
+     * Pattern for testing whether it is ALTER VIEW statement.
      */
-    private static final Pattern PATTERN_END_OF_FUNCTION = Pattern.compile(
-            "^(?:.*[\\s]+)?AS[\\s]+(['$][^\\s]*).*$", Pattern.CASE_INSENSITIVE);
+    private static final Pattern PATTERN_ALTER_VIEW = Pattern.compile(
+            "^ALTER[\\s]+VIEW[\\s]+.*$", Pattern.CASE_INSENSITIVE);
+    /**
+     * Pattern for testing whether it is COMMENT statement.
+     */
+    private static final Pattern PATTERN_COMMENT = Pattern.compile(
+            "^COMMENT[\\s]+ON[\\s]+.*$", Pattern.CASE_INSENSITIVE);
 
     /**
      * Creates a new instance of PgDumpLoader.
      */
     private PgDumpLoader() {
-        super();
     }
 
     /**
@@ -128,16 +133,13 @@ public class PgDumpLoader { //NOPMD
      *
      * @param inputStream input stream that should be read
      * @param charsetName charset that should be used to read the file
+     * @param outputIgnoredStatements whether ignored statements should be
+     * included in the output
      *
-     * @return database schema from dump fle
-     *
-     * @throws UnsupportedOperationException Thrown if unsupported encoding has
-     *         been encountered.
-     * @throws FileException Thrown if problem occured while reading input
-     *         stream.
+     * @return database schema from dump file
      */
     public static PgDatabase loadDatabaseSchema(final InputStream inputStream,
-            final String charsetName) { //NOPMD
+            final String charsetName, final boolean outputIgnoredStatements) {
 
         final PgDatabase database = new PgDatabase();
         BufferedReader reader = null;
@@ -147,7 +149,8 @@ public class PgDumpLoader { //NOPMD
                     new InputStreamReader(inputStream, charsetName));
         } catch (final UnsupportedEncodingException ex) {
             throw new UnsupportedOperationException(
-                    "Unsupported encoding: " + charsetName, ex);
+                    Resources.getString("UnsupportedEncoding") + ": "
+                    + charsetName, ex);
         }
 
         try {
@@ -163,7 +166,7 @@ public class PgDumpLoader { //NOPMD
                     continue;
                 } else if (PATTERN_CREATE_SCHEMA.matcher(line).matches()) {
                     CreateSchemaParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
                 } else if (PATTERN_DEFAULT_SCHEMA.matcher(line).matches()) {
                     final Matcher matcher =
                             PATTERN_DEFAULT_SCHEMA.matcher(line);
@@ -171,38 +174,50 @@ public class PgDumpLoader { //NOPMD
                     database.setDefaultSchema(matcher.group(1));
                 } else if (PATTERN_CREATE_TABLE.matcher(line).matches()) {
                     CreateTableParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
                 } else if (PATTERN_ALTER_TABLE.matcher(line).matches()) {
-                    AlterTableParser.parse(
-                            database, getWholeCommand(reader, line));
+                    AlterTableParser.parse(database,
+                            getWholeStatement(reader, line),
+                            outputIgnoredStatements);
                 } else if (PATTERN_CREATE_SEQUENCE.matcher(line).matches()) {
                     CreateSequenceParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
                 } else if (PATTERN_CREATE_INDEX.matcher(line).matches()) {
                     CreateIndexParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
                 } else if (PATTERN_CREATE_VIEW.matcher(line).matches()) {
                     CreateViewParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
+                } else if (PATTERN_ALTER_VIEW.matcher(line).matches()) {
+                    AlterViewParser.parse(database,
+                            getWholeStatement(reader, line),
+                            outputIgnoredStatements);
                 } else if (PATTERN_CREATE_TRIGGER.matcher(line).matches()) {
                     CreateTriggerParser.parse(
-                            database, getWholeCommand(reader, line));
+                            database, getWholeStatement(reader, line));
                 } else if (PATTERN_CREATE_FUNCTION.matcher(line).matches()) {
                     CreateFunctionParser.parse(
                             database, getWholeFunction(reader, line));
-                } else if (PATTERN_SET.matcher(line).matches()
-                        || PATTERN_COMMENT.matcher(line).matches()
-                        || PATTERN_SELECT.matcher(line).matches()
+                } else if (PATTERN_COMMENT.matcher(line).matches()) {
+                    CommentParser.parse(database,
+                            getWholeStatement(reader, line),
+                            outputIgnoredStatements);
+                } else if (PATTERN_SELECT.matcher(line).matches()
                         || PATTERN_INSERT_INTO.matcher(line).matches()
-                        || PATTERN_REVOKE.matcher(line).matches()
-                        || PATTERN_GRANT.matcher(line).matches()) {
-                    getWholeCommand(reader, line);
+                        || PATTERN_UPDATE.matcher(line).matches()
+                        || PATTERN_DELETE_FROM.matcher(line).matches()) {
+                    getWholeStatement(reader, line);
+                } else if (outputIgnoredStatements) {
+                    database.addIgnoredStatement(
+                            getWholeStatement(reader, line));
+                } else {
+                    getWholeStatement(reader, line);
                 }
 
                 line = reader.readLine();
             }
         } catch (final IOException ex) {
-            throw new FileException(FileException.CANNOT_READ_FILE, ex);
+            throw new FileException(Resources.getString("CannotReadFile"), ex);
         }
 
         return database;
@@ -213,50 +228,50 @@ public class PgDumpLoader { //NOPMD
      *
      * @param file name of file containing the dump
      * @param charsetName charset that should be used to read the file
+     * @param outputIgnoredStatements whether ignored statements should be
+     * included in the output
      *
      * @return database schema from dump file
-     *
-     * @throws FileException Thrown if file not found.
      */
     public static PgDatabase loadDatabaseSchema(final String file,
-            final String charsetName) {
+            final String charsetName, final boolean outputIgnoredStatements) {
         try {
-            return loadDatabaseSchema(new FileInputStream(file), charsetName);
+            return loadDatabaseSchema(new FileInputStream(file), charsetName,
+                    outputIgnoredStatements);
         } catch (final FileNotFoundException ex) {
-            throw new FileException("File '" + file + "' not found", ex);
+            throw new FileException(MessageFormat.format(
+                    Resources.getString("FileNotFound"), file), ex);
         }
     }
 
     /**
-     * Reads whole command from the reader into single-line string.
+     * Reads whole statement from the reader into single-line string.
      *
      * @param reader reader to be read
      * @param line first line read
      *
-     * @return whole command from the reader into single-line string
-     *
-     * @throws FileException Thrown if problem occured while reading string
-     *         from <code>reader</code>.
+     * @return whole statement from the reader into single-line string
      */
-    private static String getWholeCommand(final BufferedReader reader,
+    private static String getWholeStatement(final BufferedReader reader,
             final String line) {
         String newLine = line.trim();
-        final StringBuilder sbCommand = new StringBuilder(newLine);
+        final StringBuilder sbStatement = new StringBuilder(newLine);
 
         while (!newLine.trim().endsWith(";")) {
             try {
                 newLine = stripComment(reader.readLine()).trim();
             } catch (IOException ex) {
-                throw new FileException(FileException.CANNOT_READ_FILE, ex);
+                throw new FileException(
+                        Resources.getString("CannotReadFile"), ex);
             }
 
             if (newLine.length() > 0) {
-                sbCommand.append(' ');
-                sbCommand.append(newLine);
+                sbStatement.append(' ');
+                sbStatement.append(newLine);
             }
         }
 
-        return sbCommand.toString();
+        return sbStatement.toString();
     }
 
     /**
@@ -267,87 +282,102 @@ public class PgDumpLoader { //NOPMD
      * @param line first line read
      *
      * @return whole CREATE FUNCTION DDL from the reader into multi-line string
-     *
-     * @throws FileException Thrown if problem occured while reading string
-     *         from<code>reader</code>.
-     * @throws RuntimeException Thrown if cannot find end of function.
      */
     private static String getWholeFunction(final BufferedReader reader,
             final String line) {
         final String firstLine = line;
-        final StringBuilder sbCommand = new StringBuilder();
+        final StringBuilder sbStatement = new StringBuilder(1000);
         String newLine = line;
-        Pattern endOfFunctionPattern = null;
+        String endOfFunction = null;
+        boolean ignoreFirstOccurence = true;
         boolean searchForSemicolon = false;
+        boolean nextIsSeparator = false;
 
         while (newLine != null) {
-            if (!searchForSemicolon && (endOfFunctionPattern == null)) {
-                final Matcher matcher =
-                        PATTERN_END_OF_FUNCTION.matcher(newLine);
+            if (endOfFunction == null) {
+                boolean previousWasWhitespace = true;
 
-                if (matcher.matches()) {
-                    String endOfFunction = matcher.group(1);
+                for (int i = 0; i < newLine.length(); i++) {
+                    final char chr = newLine.charAt(i);
 
-                    if (endOfFunction.charAt(0) == '\'') {
-                        endOfFunction = "'";
+                    if (Character.isWhitespace(chr)) {
+                        previousWasWhitespace = true;
+                        continue;
+                    } else if (nextIsSeparator) {
+                        if (chr == '\'') {
+                            endOfFunction = "'";
+                        } else {
+                            endOfFunction = newLine.substring(
+                                    i, newLine.indexOf('$', i + 1) + 1);
+                        }
+
+                        break;
+                    } else if (previousWasWhitespace
+                            && Character.toUpperCase(chr) == 'A'
+                            && Character.toUpperCase(newLine.charAt(i + 1)) == 'S'
+                            && (i + 2 == newLine.length()
+                            || Character.isWhitespace(newLine.charAt(i + 2)))) {
+                        i += 2;
+                        nextIsSeparator = true;
                     } else {
-                        endOfFunction = endOfFunction.substring(
-                                0, endOfFunction.indexOf('$', 1) + 1);
+                        previousWasWhitespace = false;
                     }
-
-                    if ("'".equals(endOfFunction)) {
-                        endOfFunctionPattern = Pattern.compile(
-                                "(?:.*[^\\\\]'.*|^.*[\\s]*'[\\s]*.*$)");
-                    } else {
-                        endOfFunctionPattern = Pattern.compile(
-                                ".*\\Q" + endOfFunction + "\\E.*$",
-                                Pattern.CASE_INSENSITIVE);
-                    }
-
-                    final String stripped = newLine.replaceAll(
-                            "[\\s]+AS[\\s]+\\Q" + endOfFunction + "\\E", " ");
-                    searchForSemicolon =
-                            endOfFunctionPattern.matcher(stripped).matches();
                 }
             }
 
-            sbCommand.append(newLine);
-            sbCommand.append('\n');
+            sbStatement.append(newLine);
+            sbStatement.append('\n');
 
-            if (searchForSemicolon && newLine.trim().endsWith(";")) {
-                break;
+            if (endOfFunction != null) {
+                // count occurences
+                if (!searchForSemicolon) {
+                    int count = ignoreFirstOccurence ? -1 : 0;
+                    int pos = newLine.indexOf(endOfFunction);
+                    ignoreFirstOccurence = false;
+
+                    while (pos != -1) {
+                        count++;
+                        pos = newLine.indexOf(
+                                endOfFunction, pos + endOfFunction.length());
+                    }
+
+                    if (count % 2 == 1) {
+                        searchForSemicolon = true;
+                    }
+                }
+
+                if (searchForSemicolon && newLine.trim().endsWith(";")) {
+                    break;
+                }
             }
 
             try {
                 newLine = reader.readLine();
             } catch (final IOException ex) {
-                throw new FileException(FileException.CANNOT_READ_FILE, ex);
+                throw new FileException(
+                        Resources.getString("CannotReadFile"), ex);
             }
 
             if (newLine == null) {
                 throw new RuntimeException(
-                        "Cannot find end of function: " + firstLine);
-            }
-
-            if (!searchForSemicolon && (endOfFunctionPattern != null)
-                    && endOfFunctionPattern.matcher(newLine).matches()) {
-                searchForSemicolon = true;
+                        Resources.getString("CannotFindEndOfFunction") + ": "
+                        + firstLine);
             }
         }
 
-        return sbCommand.toString();
+        return sbStatement.toString();
     }
 
     /**
-     * Strips comment from command line.
+     * Strips comment from statement line.
      *
-     * @param command command
+     * @param statement statement
      *
-     * @return if comment was found then command without the comment, otherwise
-     *         the original command
+     * @return if comment was found then statement without the comment,
+     * otherwise the original statement
      */
-    private static String stripComment(final String command) {
-        String result = command;
+    private static String stripComment(final String statement) {
+        String result = statement;
         int pos = result.indexOf("--");
 
         while (pos >= 0) {
