@@ -1,3 +1,8 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.schema;
 
 import cz.startnet.utils.pgdiff.PgDiffUtils;
@@ -47,6 +52,14 @@ public class PgColumn {
      * Determines whether null value is allowed in the column.
      */
     private boolean nullValue = true;
+    /**
+     * Contains information about column storage type.
+     */
+    private String storage;
+    /**
+     * Comment.
+     */
+    private String comment;
 
     /**
      * Creates a new PgColumn object.
@@ -57,6 +70,24 @@ public class PgColumn {
         this.name = name;
     }
 
+    /**
+     * Getter for {@link #comment}.
+     *
+     * @return {@link #comment}
+     */
+    public String getComment() {
+        return comment;
+    }
+
+    /**
+     * Setter for {@link #comment}.
+     *
+     * @param comment {@link #comment}
+     */
+    public void setComment(final String comment) {
+        this.comment = comment;
+    }
+
     /**
      * Setter for {@link #defaultValue}.
      *
@@ -78,20 +109,18 @@ public class PgColumn {
     /**
      * Returns full definition of the column.
      *
-     * @param quoteName whether name should be quoted
      * @param addDefaults whether default value should be added in case NOT
      *        NULL constraint is specified but no default value is set
      *
      * @return full definition of the column
      */
-    public String getFullDefinition(final boolean quoteName,
-            final boolean addDefaults) {
-        final StringBuilder sbDefinition = new StringBuilder();
-        sbDefinition.append(PgDiffUtils.getQuotedName(name, quoteName));
+    public String getFullDefinition(final boolean addDefaults) {
+        final StringBuilder sbDefinition = new StringBuilder(100);
+        sbDefinition.append(PgDiffUtils.getQuotedName(name));
         sbDefinition.append(' ');
         sbDefinition.append(type);
 
-        if ((defaultValue != null) && (defaultValue.length() > 0)) {
+        if (defaultValue != null && !defaultValue.isEmpty()) {
             sbDefinition.append(" DEFAULT ");
             sbDefinition.append(defaultValue);
         } else if (!nullValue && addDefaults) {
@@ -164,6 +193,24 @@ public class PgColumn {
         return statistics;
     }
 
+    /**
+     * Getter for {@link #storage}.
+     *
+     * @return {@link #storage}
+     */
+    public String getStorage() {
+        return storage;
+    }
+
+    /**
+     * Setter for {@link #storage}.
+     *
+     * @param storage {@link #storage}
+     */
+    public void setStorage(final String storage) {
+        this.storage = storage;
+    }
+
     /**
      * Setter for {@link #type}.
      *
