@@ -1,3 +1,8 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.schema;
 
 import cz.startnet.utils.pgdiff.PgDiffUtils;
@@ -29,6 +34,10 @@ public class PgConstraint {
      * Name of the table the constraint is defined on.
      */
     private String tableName;
+    /**
+     * Comment.
+     */
+    private String comment;
 
     /**
      * Creates a new PgConstraint object.
@@ -42,23 +51,49 @@ public class PgConstraint {
     /**
      * Creates and returns SQL for creation of the constraint.
      *
-     * @param quoteNames whether names should be quoted
-     *
      * @return created SQL
      */
-    public String getCreationSQL(final boolean quoteNames) {
-        final StringBuilder sbSQL = new StringBuilder();
+    public String getCreationSQL() {
+        final StringBuilder sbSQL = new StringBuilder(100);
         sbSQL.append("ALTER TABLE ");
-        sbSQL.append(PgDiffUtils.getQuotedName(getTableName(), quoteNames));
+        sbSQL.append(PgDiffUtils.getQuotedName(getTableName()));
         sbSQL.append("\n\tADD CONSTRAINT ");
-        sbSQL.append(PgDiffUtils.getQuotedName(getName(), quoteNames));
+        sbSQL.append(PgDiffUtils.getQuotedName(getName()));
         sbSQL.append(' ');
         sbSQL.append(getDefinition());
         sbSQL.append(';');
 
+        if (comment != null && !comment.isEmpty()) {
+            sbSQL.append("\n\nCOMMENT ON CONSTRAINT ");
+            sbSQL.append(PgDiffUtils.getQuotedName(name));
+            sbSQL.append(" ON ");
+            sbSQL.append(PgDiffUtils.getQuotedName(tableName));
+            sbSQL.append(" IS ");
+            sbSQL.append(comment);
+            sbSQL.append(';');
+        }
+
         return sbSQL.toString();
     }
 
+    /**
+     * Getter for {@link #comment}.
+     *
+     * @return {@link #comment}
+     */
+    public String getComment() {
+        return comment;
+    }
+
+    /**
+     * Setter for {@link #comment}.
+     *
+     * @param comment {@link #comment}
+     */
+    public void setComment(final String comment) {
+        this.comment = comment;
+    }
+
     /**
      * Setter for {@link #definition}.
      *
@@ -80,16 +115,14 @@ public class PgConstraint {
     /**
      * Creates and returns SQL for dropping the constraint.
      *
-     * @param quoteNames whether names should be quoted
-     *
      * @return created SQL
      */
-    public String getDropSQL(final boolean quoteNames) {
-        final StringBuilder sbSQL = new StringBuilder();
+    public String getDropSQL() {
+        final StringBuilder sbSQL = new StringBuilder(100);
         sbSQL.append("ALTER TABLE ");
-        sbSQL.append(PgDiffUtils.getQuotedName(getTableName(), quoteNames));
+        sbSQL.append(PgDiffUtils.getQuotedName(getTableName()));
         sbSQL.append("\n\tDROP CONSTRAINT ");
-        sbSQL.append(PgDiffUtils.getQuotedName(getName(), quoteNames));
+        sbSQL.append(PgDiffUtils.getQuotedName(getName()));
         sbSQL.append(';');
 
         return sbSQL.toString();
@@ -156,9 +189,9 @@ public class PgConstraint {
             equals = true;
         } else if (object instanceof PgConstraint) {
             final PgConstraint constraint = (PgConstraint) object;
-            equals = definition.equals(constraint.definition)
-                    && name.equals(constraint.name)
-                    && tableName.equals(constraint.tableName);
+            equals = definition.equals(constraint.getDefinition())
+                    && name.equals(constraint.getName())
+                    && tableName.equals(constraint.getTableName());
         }
 
         return equals;
