@@ -1,8 +1,14 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.schema;
 
 import cz.startnet.utils.pgdiff.PgDiffUtils;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 
 /**
@@ -15,36 +21,50 @@ public class PgTable {
     /**
      * List of columns defined on the table.
      */
+    @SuppressWarnings("CollectionWithoutInitialCapacity")
     private final List<PgColumn> columns = new ArrayList<PgColumn>();
     /**
      * List of constraints defined on the table.
      */
+    @SuppressWarnings("CollectionWithoutInitialCapacity")
     private final List<PgConstraint> constraints =
             new ArrayList<PgConstraint>();
     /**
      * List of indexes defined on the table.
      */
+    @SuppressWarnings("CollectionWithoutInitialCapacity")
     private final List<PgIndex> indexes = new ArrayList<PgIndex>();
     /**
      * List of triggers defined on the table.
      */
+    @SuppressWarnings("CollectionWithoutInitialCapacity")
     private final List<PgTrigger> triggers = new ArrayList<PgTrigger>();
     /**
      * Name of the index on which the table is clustered
      */
     private String clusterIndexName;
     /**
-     * Definition of names of inherited tables.
+     * List of names of inherited tables.
      */
-    private String inherits;
+    @SuppressWarnings("CollectionWithoutInitialCapacity")
+    private final List<String> inherits = new ArrayList<String>();
     /**
      * Name of the table.
      */
     private String name;
     /**
-     * Whether WITH OIDS is used.
+     * WITH clause. If value is null then it is not set, otherwise can be set to
+     * OIDS=true, OIDS=false, or storage parameters can be set.
      */
-    private boolean withOIDS;
+    private String with;
+    /**
+     * Tablespace value.
+     */
+    private String tablespace;
+    /**
+     * Comment.
+     */
+    private String comment;
 
     /**
      * Creates a new PgTable object.
@@ -81,26 +101,40 @@ public class PgTable {
      * @return found column or null if no such column has been found
      */
     public PgColumn getColumn(final String name) {
-        PgColumn column = null;
-
-        for (PgColumn curColumn : columns) {
-            if (curColumn.getName().equals(name)) {
-                column = curColumn;
-
-                break;
+        for (PgColumn column : columns) {
+            if (column.getName().equals(name)) {
+                return column;
             }
         }
 
-        return column;
+        return null;
     }
 
     /**
-     * Getter for {@link #columns}.
+     * Getter for {@link #columns}. The list cannot be modified.
      *
      * @return {@link #columns}
      */
     public List<PgColumn> getColumns() {
-        return columns;
+        return Collections.unmodifiableList(columns);
+    }
+
+    /**
+     * Getter for {@link #comment}.
+     *
+     * @return {@link #comment}
+     */
+    public String getComment() {
+        return comment;
+    }
+
+    /**
+     * Setter for {@link #comment}.
+     *
+     * @param comment {@link #comment}
+     */
+    public void setComment(final String comment) {
+        this.comment = comment;
     }
 
     /**
@@ -112,81 +146,127 @@ public class PgTable {
      * @return found constraint or null if no such constraint has been found
      */
     public PgConstraint getConstraint(final String name) {
-        PgConstraint constraint = null;
-
-        for (PgConstraint curConstraint : constraints) {
-            if (curConstraint.getName().equals(name)) {
-                constraint = curConstraint;
-
-                break;
+        for (PgConstraint constraint : constraints) {
+            if (constraint.getName().equals(name)) {
+                return constraint;
             }
         }
 
-        return constraint;
+        return null;
     }
 
     /**
-     * Getter for {@link #constraints}.
+     * Getter for {@link #constraints}. The list cannot be modified.
      *
      * @return {@link #constraints}
      */
     public List<PgConstraint> getConstraints() {
-        return constraints;
+        return Collections.unmodifiableList(constraints);
     }
 
     /**
      * Creates and returns SQL for creation of the table.
      *
-     * @param quoteNames whether names should be quoted
-     *
-     * @return created SQL command
+     * @return created SQL statement
      */
-    public String getCreationSQL(final boolean quoteNames) {
-        final StringBuilder sbSQL = new StringBuilder();
+    public String getCreationSQL() {
+        final StringBuilder sbSQL = new StringBuilder(1000);
         sbSQL.append("CREATE TABLE ");
-        sbSQL.append(PgDiffUtils.getQuotedName(name, quoteNames));
+        sbSQL.append(PgDiffUtils.getQuotedName(name));
         sbSQL.append(" (\n");
 
         for (PgColumn column : columns) {
             sbSQL.append("\t");
-            sbSQL.append(column.getFullDefinition(quoteNames, false));
+            sbSQL.append(column.getFullDefinition(false));
             sbSQL.append(",\n");
         }
 
         sbSQL.setLength(sbSQL.length() - 2);
         sbSQL.append("\n)");
 
-        if ((inherits != null) && (inherits.length() > 0)) {
-            sbSQL.append("\nINHERITS ");
-            sbSQL.append(inherits);
+        if (inherits != null && !inherits.isEmpty()) {
+            sbSQL.append("\nINHERITS (");
+
+            boolean first = true;
+
+            for (final String tableName : inherits) {
+                if (first) {
+                    first = false;
+                } else {
+                    sbSQL.append(", ");
+                }
+
+                sbSQL.append(tableName);
+            }
+
+            sbSQL.append(")");
+        }
+
+        if (with != null && !with.isEmpty()) {
+            sbSQL.append("\n");
+
+            if ("OIDS=false".equalsIgnoreCase(with)) {
+                sbSQL.append("WITHOUT OIDS");
+            } else {
+                sbSQL.append("WITH ");
+
+                if ("OIDS".equalsIgnoreCase(with)
+                        || "OIDS=true".equalsIgnoreCase(with)) {
+                    sbSQL.append("OIDS");
+                } else {
+                    sbSQL.append(with);
+                }
+            }
+        }
+
+        if (tablespace != null && !tablespace.isEmpty()) {
+            sbSQL.append("\nTABLESPACE ");
+            sbSQL.append(tablespace);
         }
 
         sbSQL.append(';');
 
         for (PgColumn column : getColumnsWithStatistics()) {
             sbSQL.append("\nALTER TABLE ONLY ");
-            sbSQL.append(PgDiffUtils.getQuotedName(name, quoteNames));
+            sbSQL.append(PgDiffUtils.getQuotedName(name));
             sbSQL.append(" ALTER COLUMN ");
             sbSQL.append(
-                    PgDiffUtils.getQuotedName(column.getName(), quoteNames));
+                    PgDiffUtils.getQuotedName(column.getName()));
             sbSQL.append(" SET STATISTICS ");
             sbSQL.append(column.getStatistics());
             sbSQL.append(';');
         }
 
+        if (comment != null && !comment.isEmpty()) {
+            sbSQL.append("\n\nCOMMENT ON TABLE ");
+            sbSQL.append(PgDiffUtils.getQuotedName(name));
+            sbSQL.append(" IS ");
+            sbSQL.append(comment);
+            sbSQL.append(';');
+        }
+
+        for (final PgColumn column : columns) {
+            if (column.getComment() != null && !column.getComment().isEmpty()) {
+                sbSQL.append("\n\nCOMMENT ON COLUMN ");
+                sbSQL.append(PgDiffUtils.getQuotedName(name));
+                sbSQL.append('.');
+                sbSQL.append(PgDiffUtils.getQuotedName(column.getName()));
+                sbSQL.append(" IS ");
+                sbSQL.append(column.getComment());
+                sbSQL.append(';');
+            }
+        }
+
         return sbSQL.toString();
     }
 
     /**
-     * Creates and returns SQL command for dropping the table.
+     * Creates and returns SQL statement for dropping the table.
      *
-     * @param quoteNames whether names should be quoted
-     *
-     * @return created SQL command
+     * @return created SQL statement
      */
-    public String getDropSQL(final boolean quoteNames) {
-        return "DROP TABLE " + PgDiffUtils.getQuotedName(getName(), quoteNames)
-                + ";";
+    public String getDropSQL() {
+        return "DROP TABLE " + PgDiffUtils.getQuotedName(getName()) + ";";
     }
 
     /**
@@ -197,35 +277,48 @@ public class PgTable {
      * @return found index or null if no such index has been found
      */
     public PgIndex getIndex(final String name) {
-        PgIndex index = null;
-
-        for (PgIndex curIndex : indexes) {
-            if (curIndex.getName().equals(name)) {
-                index = curIndex;
-
-                break;
+        for (PgIndex index : indexes) {
+            if (index.getName().equals(name)) {
+                return index;
             }
         }
 
-        return index;
+        return null;
     }
 
     /**
-     * Getter for {@link #indexes}.
+     * Finds trigger according to specified trigger <code>name</code>.
+     *
+     * @param name name of the trigger to be searched
+     *
+     * @return found trigger or null if no such trigger has been found
+     */
+    public PgTrigger getTrigger(final String name) {
+        for (PgTrigger trigger : triggers) {
+            if (trigger.getName().equals(name)) {
+                return trigger;
+            }
+        }
+
+        return null;
+    }
+
+    /**
+     * Getter for {@link #indexes}. The list cannot be modified.
      *
      * @return {@link #indexes}
      */
     public List<PgIndex> getIndexes() {
-        return indexes;
+        return Collections.unmodifiableList(indexes);
     }
 
     /**
      * Setter for {@link #inherits}.
      *
-     * @param inherits {@link #inherits}
+     * @param tableName name of inherited table
      */
-    public void setInherits(final String inherits) {
-        this.inherits = inherits;
+    public void addInherits(final String tableName) {
+        inherits.add(tableName);
     }
 
     /**
@@ -233,8 +326,8 @@ public class PgTable {
      *
      * @return {@link #inherits}
      */
-    public String getInherits() {
-        return inherits;
+    public List<String> getInherits() {
+        return Collections.unmodifiableList(inherits);
     }
 
     /**
@@ -256,30 +349,48 @@ public class PgTable {
     }
 
     /**
-     * Getter for {@link #triggers}.
+     * Getter for {@link #triggers}. The list cannot be modified.
      *
      * @return {@link #triggers}
      */
     public List<PgTrigger> getTriggers() {
-        return triggers;
+        return Collections.unmodifiableList(triggers);
     }
 
     /**
-     * Setter for {@link #withOIDS}.
+     * Setter for {@link #with}.
      *
-     * @param withOIDS {@link #withOIDS}
+     * @param with {@link #with}
      */
-    public void setWithOIDS(final boolean withOIDS) {
-        this.withOIDS = withOIDS;
+    public void setWith(final String with) {
+        this.with = with;
     }
 
     /**
-     * Getter for {@link #withOIDS}
+     * Getter for {@link #with}
      *
-     * @return {@link #withOIDS}
+     * @return {@link #with}
      */
-    public boolean isWithOIDS() {
-        return withOIDS;
+    public String getWith() {
+        return with;
+    }
+
+    /**
+     * Getter for {@link #tablespace}.
+     *
+     * @return {@link #tablespace}
+     */
+    public String getTablespace() {
+        return tablespace;
+    }
+
+    /**
+     * Setter for {@link #tablespace}.
+     *
+     * @param tablespace {@link #tablespace}
+     */
+    public void setTablespace(final String tablespace) {
+        this.tablespace = tablespace;
     }
 
     /**
@@ -328,17 +439,13 @@ public class PgTable {
      *         false
      */
     public boolean containsColumn(final String name) {
-        boolean found = false;
-
         for (PgColumn column : columns) {
             if (column.getName().equals(name)) {
-                found = true;
-
-                break;
+                return true;
             }
         }
 
-        return found;
+        return false;
     }
 
     /**
@@ -351,17 +458,13 @@ public class PgTable {
      *         otherwise false
      */
     public boolean containsConstraint(final String name) {
-        boolean found = false;
-
         for (PgConstraint constraint : constraints) {
             if (constraint.getName().equals(name)) {
-                found = true;
-
-                break;
+                return true;
             }
         }
 
-        return found;
+        return false;
     }
 
     /**
@@ -374,17 +477,13 @@ public class PgTable {
      *         false
      */
     public boolean containsIndex(final String name) {
-        boolean found = false;
-
         for (PgIndex index : indexes) {
             if (index.getName().equals(name)) {
-                found = true;
-
-                break;
+                return true;
             }
         }
 
-        return found;
+        return false;
     }
 
     /**
@@ -393,6 +492,7 @@ public class PgTable {
      * @return list of columns that have statistics defined
      */
     private List<PgColumn> getColumnsWithStatistics() {
+        @SuppressWarnings("CollectionWithoutInitialCapacity")
         final List<PgColumn> list = new ArrayList<PgColumn>();
 
         for (PgColumn column : columns) {
