@@ -1,3 +1,8 @@
+/**
+ * Copyright 2006 StartNet s.r.o.
+ *
+ * Distributed under MIT license
+ */
 package cz.startnet.utils.pgdiff.loader;
 
 import org.junit.Test;
@@ -46,7 +51,16 @@ public class PgDumpLoaderTest {
                     {3},
                     {4},
                     {5},
-                    {6}
+                    {6},
+                    {7},
+                    {8},
+                    {9},
+                    {10},
+                    {11},
+                    {12},
+                    {13},
+                    {14},
+                    {15}
                 });
     }
 
@@ -57,6 +71,6 @@ public class PgDumpLoaderTest {
     public void loadSchema() {
         PgDumpLoader.loadDatabaseSchema(
                 getClass().getResourceAsStream("schema_" + fileIndex + ".sql"),
-                "UTF-8");
+                "UTF-8", false);
     }
 }
